@@ -1,22 +1,10 @@
-#!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
 """
-Copyright (C) 2009-2014 Wolfgang Rohdewald <wolfgang@rohdewald.de>
+Copyright (C) 2009-2016 Wolfgang Rohdewald <wolfgang@rohdewald.de>
 
-Kajongg is free software you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation either version 2 of the License, or
-(at your option) any later version.
+SPDX-License-Identifier: GPL-2.0
 
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this program if not, write to the Free Software
-Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
 The DBPasswordChecker is based on an example from the book
 Twisted Network Programming Essentials by Abe Fettig, 2006
@@ -28,14 +16,18 @@ O'Reilly Media, Inc., ISBN 0-596-10032-9
 import sys
 import os
 import logging
-from signal import signal, SIGABRT, SIGINT, SIGTERM
+import datetime
+import argparse
+from typing import TYPE_CHECKING, Tuple, Any, Optional, Sequence, List, Mapping, Dict, Union, cast, Type
 
 from zope.interface import implementer
 
 
-def cleanExit(*dummyArgs):
+def cleanExit(*unusedArgs: Any) ->None:
     """we want to cleanly close sqlite3 files"""
-    if Options.socket and os.name != 'nt':
+    if Debug.quit:
+        logDebug('cleanExit')
+    if Options.socket and sys.platform != 'win32':
         if os.path.exists(Options.socket):
             os.remove(Options.socket)
     try:
@@ -48,37 +40,37 @@ def cleanExit(*dummyArgs):
     except NameError:
         logging.shutdown()
     try:
-        reactor.stop()
+        reactor.stop()  # type:ignore[misc]
     except NameError:
         sys.exit(0)
     except ReactorNotRunning:
         pass
 
-signal(SIGABRT, cleanExit)
-signal(SIGINT, cleanExit)
-signal(SIGTERM, cleanExit)
-if os.name != 'nt':
-    from signal import SIGHUP, SIGQUIT
-    signal(SIGHUP, cleanExit)
-    signal(SIGQUIT, cleanExit)
-
+from common import handleSignals
+handleSignals(cleanExit)
 
 from common import Options, Internal, Debug
 Internal.isServer = True
 Internal.logPrefix = 'S'
 
 from twisted.spread import pb
+from twisted.internet.interfaces import IReactorCore
 from twisted.internet import error
-from twisted.internet.defer import maybeDeferred, fail, succeed
+from twisted.internet.defer import maybeDeferred, fail, succeed, Deferred
 from twisted.cred import checkers, portal, credentials, error as credError
-from twisted.internet import reactor
+from twisted.internet import reactor as reactor_module
 from twisted.internet.error import ReactorNotRunning
-reactor.addSystemEventTrigger('before', 'shutdown', cleanExit)
-Internal.reactor = reactor
+if TYPE_CHECKING:
+    from twisted.python.failure import Failure
+
+reactor = cast(IReactorCore, reactor_module)
+reactor.addSystemEventTrigger('before', 'shutdown', cleanExit)  # type:ignore[arg-type]
+Internal.reactor = reactor  # type:ignore[assignment]
 
 from player import Players
 from query import Query, initDb
-from log import m18n, m18nE, logDebug, logWarning, logError, SERVERMARK
+from log import logDebug, logWarning, logError, logInfo, logException, SERVERMARK
+from mi18n import i18n, i18nE
 from util import elapsedSince
 from message import Message, ChatMessage
 from deferredutil import DeferredBlock
@@ -89,73 +81,72 @@ from servertable import ServerTable, ServerGame
 
 
 @implementer(checkers.ICredentialsChecker)
-class DBPasswordChecker(object):
+class DBPasswordChecker:
 
     """checks against our sqlite3 databases"""
     credentialInterfaces = (credentials.IUsernamePassword,
                             credentials.IUsernameHashedPassword)
 
-    def requestAvatarId(self, cred):  # pylint: disable=no-self-use
+    def requestAvatarId(self, cred: pb._PortalAuthChallenger) ->Deferred:
         """get user id from database"""
         cred.username = cred.username.decode('utf-8')
         args = cred.username.split(SERVERMARK)
         if len(args) > 1:
-            if args[0] == u'adduser':
+            if args[0] == 'adduser':
                 cred.username = args[1]
                 password = args[2]
                 query = Query(
                     'insert or ignore into player(name,password) values(?,?)',
                     (cred.username,
                      password))
-            elif args[1] == u'deluser':
+            elif args[1] == 'deluser':
                 pass
         query = Query(
             'select id, password from player where name=?', (cred.username,))
-        if not len(query.records):
-            template = u'Wrong username: %1'
+        if not query.records:
+            template = 'Wrong username: %1'
             if Debug.connections:
-                logDebug(m18n(template, cred.username))
+                logDebug(i18n(template, cred.username))
             return fail(credError.UnauthorizedLogin(srvMessage(template, cred.username)))
         userid, password = query.records[0]
-        # checkPassword uses md5 which cannot handle unicode strings (python
-        # 2.7)
         defer1 = maybeDeferred(cred.checkPassword, password.encode('utf-8'))
-        defer1.addCallback(DBPasswordChecker._checkedPassword, userid)
+        defer1.addCallback(DBPasswordChecker._checkedPassword, userid).addErrback(logException)
         return defer1
 
     @staticmethod
-    def _checkedPassword(matched, userid):
-        """after the password has been checked"""
+    def _checkedPassword(matched: bool, userid:str) ->Union[str, Deferred]:
+        """after the password has been checked. TODO: is the result type correct?"""
         if not matched:
-            return fail(credError.UnauthorizedLogin(srvMessage(m18nE('Wrong password'))))
+            return fail(credError.UnauthorizedLogin(srvMessage(i18nE('Wrong password'))))
         return userid
 
 
-class MJServer(object):
+class MJServer:
 
     """the real mah jongg server"""
 
-    def __init__(self):
-        self.tables = {}
-        self.srvUsers = list()
+    def __init__(self) ->None:
+        self.tables:Dict[int, ServerTable] = {}
+        self.srvUsers:List[User] = []
         Players.load()
-        self.lastPing = None
+        self.lastPing = datetime.datetime.now()
         self.checkPings()
 
-    def chat(self, chatString):
+    def chat(self, chatString:str) ->None:
         """a client sent us a chat message"""
-        chatLine = ChatMessage(chatString)
-        if Debug.chat:
-            logDebug(u'server got chat message %s' % chatLine)
+        # FIXME: why does this work?
+        chatLine = ChatMessage(chatString)  # type:ignore[arg-type]
+
+
         self.tables[chatLine.tableid].sendChatMessage(chatLine)
 
-    def login(self, user):
+    def login(self, user: User) ->None:
         """accept a new user"""
         if user not in self.srvUsers:
             self.srvUsers.append(user)
             self.loadSuspendedTables(user)
 
-    def callRemote(self, user, *args, **kwargs):
+    def callRemote(self, user:User, *args: Any, **kwargs:Mapping[Any, Any]) ->Deferred:
         """if we still have a connection, call remote, otherwise clean up"""
         if user.mind:
             try:
@@ -164,118 +155,120 @@ class MJServer(object):
             except (pb.DeadReferenceError, pb.PBConnectionLost):
                 user.mind = None
                 self.logout(user)
+        return succeed([])
 
     @staticmethod
-    def __stopAfterLastDisconnect():
+    def __stopAfterLastDisconnect() ->None:
         """as the name says"""
         if Options.socket and not Options.continueServer:
             try:
-                reactor.stop()
+                reactor.stop()  # type:ignore[misc]
                 if Debug.connections:
-                    logDebug(u'local server terminates from %s. Reason: last client disconnected' % (
-                        Options.socket))
+                    logDebug(f'local server terminates from {Options.socket}. Reason: last client disconnected')
             except ReactorNotRunning:
                 pass
 
-    def checkPings(self):
+    def checkPings(self) ->None:
         """are all clients still alive? If not log them out"""
-        if not self.srvUsers and self.lastPing and elapsedSince(self.lastPing) > 30:
+        since = elapsedSince(self.lastPing)
+        if self.srvUsers and since > 30:
+            if Debug.quit:
+                logDebug(f'no ping since {elapsedSince(self.lastPing)} seconds but we still have users:{self.srvUsers}')
+        if not self.srvUsers and since > 30:
             # no user at all since 30 seconds, but we did already have a user
             self.__stopAfterLastDisconnect()
         for user in self.srvUsers:
-            self.lastPing = max(
-                self.lastPing,
-                user.lastPing) if self.lastPing else user.lastPing
             if elapsedSince(user.lastPing) > 60:
-                logDebug(
-                    u'No messages from %s since 60 seconds, clearing connection now' %
-                    user.name)
+                logInfo(
+                    f'No messages from {user.name} since 60 seconds, clearing connection now')
                 user.mind = None
                 self.logout(user)
         reactor.callLater(10, self.checkPings)
 
     @staticmethod
-    def ignoreLostConnection(failure):
+    def ignoreLostConnection(failure: 'Failure') ->None:
         """if the client went away correctly, do not dump error messages on stdout."""
         msg = failure.getErrorMessage()
         if 'twisted.internet.error.ConnectionDone' not in msg:
             logError(msg)
         failure.trap(pb.PBConnectionLost)
 
-    def sendTables(self, user, tables=None):
+    def sendTables(self, user:User, tables:Optional[List[ServerTable]]=None) ->Deferred:
         """send tables to user. If tables is None, he gets all new tables and those
         suspended tables he was sitting on"""
         if tables is None:
-            tables = list(x for x in self.tables.values()
-                          if not x.running and (not x.suspendedAt or x.hasName(user.name)))
-        if len(tables):
-            data = list(x.asSimpleList() for x in tables)
+            tables = [
+                x for x in self.tables.values()
+                if not x.running and (not x.suspendedAt or x.hasName(user.name))]
+        if tables:
+            data = [x.asSimpleList() for x in tables]
             if Debug.table:
                 logDebug(
-                    u'sending %d tables to %s: %s' %
-                    (len(tables), user.name, data))
+                    f'sending {len(tables)} tables to {user.name}: {data}')
             return self.callRemote(user, 'newTables', data)
-        else:
-            return succeed([])
+        return succeed([])
 
-    def _lookupTable(self, tableid):
+    def _lookupTable(self, tableid: int) ->ServerTable:
         """return table by id or raise exception"""
         if tableid not in self.tables:
-            raise srvError(
+            srvError(
                 pb.Error,
-                m18nE('table with id <numid>%1</numid> not found'),
+                i18nE('table with id <numid>%1</numid> not found'),
                 tableid)
         return self.tables[tableid]
 
-    def generateTableId(self):
+    def generateTableId(self) ->int:
         """generates a new table id: the first free one"""
         usedIds = set(self.tables or [0])
         availableIds = set(x for x in range(1, 2 + max(usedIds)))
         return min(availableIds - usedIds)
 
-    def newTable(self, user, ruleset, playOpen,
-                 autoPlay, wantedGame, tableId=None):
+    def newTable(self, user:User, ruleset:str, playOpen:bool,
+                 autoPlay:bool, wantedGame:str, tableId:Optional[int]=None) ->Optional[Deferred]:
         """user creates new table and joins it"""
-        def gotRuleset(ruleset):
+        def gotRuleset(ruleset:str) ->None:
             """now we have the full ruleset definition from the client"""
             Ruleset.cached(
                 ruleset).save()  # make it known to the cache and save in db
         if tableId in self.tables:
+            assert tableId is not None  # for mypy
             return fail(srvError(pb.Error,
-                                 'You want a new table with id=%d but that id is already used for table %s' % (
-                                     tableId, self.tables[tableId])))
+                                 f'You want a new table with id={int(tableId)} '
+                                 f'but that id is already used for table {self.tables[tableId]}'))
         if Ruleset.hashIsKnown(ruleset):
             return self.__newTable(None, user, ruleset, playOpen, autoPlay, wantedGame, tableId)
-        else:
-            return self.callRemote(user, 'needRuleset', ruleset).addCallback(
-                gotRuleset).addCallback(
-                    self.__newTable, user, ruleset, playOpen, autoPlay, wantedGame, tableId)
+        _ = self.callRemote(user, 'needRuleset', ruleset)
+        if _:
+            return _.addCallback(
+                gotRuleset).addErrback(logException).addCallback(
+                    self.__newTable, user, ruleset, playOpen, autoPlay, wantedGame, tableId).addErrback(logException)
+        return None
 
-    def __newTable(self, dummy, user, ruleset,
-                   playOpen, autoPlay, wantedGame, tableId=None):
+    def __newTable(self, unused:None, user:User, ruleset:str,
+                   playOpen:bool, autoPlay:bool, wantedGame:str, tableId:Optional[int]=None) ->Deferred:
         """now we know the ruleset"""
-        def sent(dummy):
+        def sent(unused: Any) ->int:
             """new table sent to user who created it"""
             return table.tableid
         table = ServerTable(
             self,
-            user,
-            ruleset,
-            None,
-            playOpen,
-            autoPlay,
-            wantedGame,
-            tableId)
+            owner=user,
+            ruleset=ruleset,
+            suspendedAt=None,
+            playOpen=playOpen,
+            autoPlay=autoPlay,
+            wantedGame=wantedGame,
+            tableId=tableId)
         result = None
         for srvUser in self.srvUsers:
             deferred = self.sendTables(srvUser, [table])
             if user == srvUser:
                 result = deferred
-                deferred.addCallback(sent)
+                deferred.addCallback(sent).addErrback(logException)
         assert result
         return result
 
-    def needRulesets(self, rulesetHashes):
+    def needRulesets(self, rulesetHashes: List[str]) -> List[List[List[Union[int, str, float]]]]:
         """the client wants those full rulesets"""
         result = []
         for table in self.tables.values():
@@ -283,11 +276,11 @@ class MJServer(object):
                 result.append(table.ruleset.toList())
         return result
 
-    def joinTable(self, user, tableid):
+    def joinTable(self, user: User, tableid: int) ->bool:
         """user joins table"""
         table = self._lookupTable(tableid)
         table.addUser(user)
-        block = DeferredBlock(table)
+        block = DeferredBlock(table, where='joinTable')
         block.tell(
             None,
             self.srvUsers,
@@ -295,21 +288,22 @@ class MJServer(object):
             source=table.asSimpleList())
         if len(table.users) == table.maxSeats():
             if Debug.table:
-                logDebug(u'Table %s: All seats taken, starting' % table)
+                logDebug(f'Table {table}: All seats taken, starting')
 
-            def startTable(dummy):
+            def startTable(unused: Any) -> None:
                 """now all players know about our join"""
+                assert table.owner
                 table.readyForGameStart(table.owner)
             block.callback(startTable)
         else:
             block.callback(False)
         return True
 
-    def tablesWith(self, user):
+    def tablesWith(self, user: User) ->List[int]:
         """table ids with user, except table 'without'"""
-        return list(x.tableid for x in self.tables.values() if user in x.users)
+        return [x.tableid for x in self.tables.values() if user in x.users]
 
-    def leaveTable(self, user, tableid, message=None, *args):
+    def leaveTable(self, user: User, tableid: int, message: str, *args: str) ->bool:
         """user leaves table. If no human user is left on a new table, remove it"""
         if tableid in self.tables:
             table = self.tables[tableid]
@@ -321,7 +315,7 @@ class MJServer(object):
                 else:
                     table.delUser(user)
                     if self.srvUsers:
-                        block = DeferredBlock(table)
+                        block = DeferredBlock(table, where='leaveTable')
                         block.tell(
                             None,
                             self.srvUsers,
@@ -330,19 +324,17 @@ class MJServer(object):
                         block.callback(False)
         return True
 
-    def startGame(self, user, tableid):
+    def startGame(self, user: User, tableid: int) ->None:
         """try to start the game"""
-        return self._lookupTable(tableid).readyForGameStart(user)
+        self._lookupTable(tableid).readyForGameStart(user)
 
-    def removeTable(self, table, reason, message=None, *args):
+    def removeTable(self, table: ServerTable, reason: str, message: str, *args:Union[str, int]) ->None:
         """remove a table"""
         assert reason in ('silent', 'tableRemoved', 'gameOver', 'abort')
         # HumanClient implements methods remote_tableRemoved etc.
-        message = message or ''
         if Debug.connections or reason == 'abort':
             logDebug(
-                u'%s%s ' % (('%s:' % table.game.seed) if table.game else '',
-                            m18n(message, *args)), withGamePrefix=None)
+                f"{f'{int(table.game.seed)}:' if table.game else ''}{i18n(message, *args)} ", withGamePrefix=False)
         if table.tableid in self.tables:
             del self.tables[table.tableid]
             if reason == 'silent':
@@ -356,12 +348,11 @@ class MJServer(object):
                 table.delUser(user)
             if Debug.table:
                 logDebug(
-                    u'removing table %d: %s %s' %
-                    (table.tableid, m18n(message, *args), reason))
+                    f'removing table {int(table.tableid)}: {i18n(message, *args)} {reason}')
         if table.game:
             table.game.close()
 
-    def logout(self, user):
+    def logout(self, user: User) ->None:
         """remove user from all tables"""
         if user not in self.srvUsers:
             return
@@ -370,22 +361,22 @@ class MJServer(object):
             self.leaveTable(
                 user,
                 tableid,
-                m18nE('Player %1 has logged out'),
+                i18nE('Player %1 has logged out'),
                 user.name)
         # wait a moment. We want the leaveTable message to arrive everywhere before
         # we say serverDisconnects. Sometimes the order was reversed.
         reactor.callLater(1, self.__logout2, user)
 
-    def __logout2(self, user):
+    def __logout2(self, user: User) ->None:
         """now the leaveTable message had a good chance to get to the clients first"""
         self.callRemote(user, 'serverDisconnects')
         user.mind = None
         for block in DeferredBlock.blocks:
             for request in block.requests:
                 if request.user == user:
-                    block.removeRequest(request)
+                    request.answer = Message.Abort
 
-    def loadSuspendedTables(self, user):
+    def loadSuspendedTables(self, user: User) ->None:
         """loads all yet unloaded suspended tables where this
         user is participating. We do not unload them if the
         user logs out, there are filters anyway returning only
@@ -411,18 +402,19 @@ class MJServer(object):
                 table = ServerTable(
                     self, None, ruleset, suspendTime, playOpen=False,
                     autoPlay=False, wantedGame=str(seed))
-                table.game = ServerGame.loadFromDB(gameid)
+                table.game = cast(ServerGame, ServerGame.loadFromDB(gameid))
 
 
 @implementer(portal.IRealm)
-class MJRealm(object):
+class MJRealm:
 
     """connects mind and server"""
 
-    def __init__(self):
-        self.server = None
+    def __init__(self) ->None:
+        self.server : Optional[MJServer] = None
 
-    def requestAvatar(self, avatarId, mind, *interfaces):
+    def requestAvatar(self, avatarId: str, mind: pb.RemoteReference,
+        *interfaces: Sequence[pb.IPerspective]) -> Tuple[Type[pb.IPerspective], User, Any]:
         """as the tutorials do..."""
         if pb.IPerspective not in interfaces:
             raise NotImplementedError("No supported avatar interface")
@@ -430,91 +422,78 @@ class MJRealm(object):
         avatar.server = self.server
         avatar.attached(mind)
         if Debug.connections:
-            logDebug(u'Connection from %s ' % avatar.source())
+            logDebug(f'Connection from {avatar.source()} ')
         return pb.IPerspective, avatar, lambda a=avatar: a.detached(mind)
 
-
-def parseArgs():
+def parseArgs() -> argparse.Namespace:
     """as the name says"""
-    from optparse import OptionParser
-    parser = OptionParser()
+    parser = argparse.ArgumentParser()
     defaultPort = Internal.defaultPort
-    parser.add_option('', '--port', dest='port',
-                      help=m18n(
-                          'the server will listen on PORT (%d)' %
-                          defaultPort),
+    parser.add_argument('--port', dest='port',
+                      help=i18n(
+                          f'the server will listen on PORT ({int(defaultPort)})'),
                       type=int, default=defaultPort)
-    parser.add_option('', '--socket', dest='socket',
-                      help=m18n('the server will listen on SOCKET'), default=None)
-    parser.add_option(
-        '',
+    parser.add_argument('--socket', dest='socket',
+                      help=i18n('the server will listen on SOCKET'), default=None)
+    parser.add_argument(
         '--db',
         dest='dbpath',
-        help=m18n('name of the database'),
+        help=i18n('name of the database'),
         default=None)
-    parser.add_option(
-        '', '--continue', dest='continueServer', action='store_true',
-        help=m18n('do not terminate local game server after last client disconnects'), default=False)
-    parser.add_option('', '--debug', dest='debug',
+    parser.add_argument(
+        '--continue', dest='continueServer', action='store_true',
+        help=i18n('do not terminate local game server after last client disconnects'), default=False)
+    parser.add_argument('--debug', dest='debug',
                       help=Debug.help())
-    parser.add_option('', '--nokde', dest='nokde', action='store_true',
-                      help=m18n('do not use KDE bindings. Only for testing'))
-    parser.add_option('', '--qt5', dest='qt5', action='store_true',
-                      help=m18n('Force using Qt5. Currently Qt4 is used by default'))
-    (options, args) = parser.parse_args()
-    if args and ''.join(args):
-        logWarning(m18n('unrecognized arguments:%1', ' '.join(args)))
-        sys.exit(2)
-    Options.continueServer |= options.continueServer
-    if options.dbpath:
-        Options.dbPath = os.path.expanduser(options.dbpath)
-    if options.socket:
-        Options.socket = options.socket
-    Debug.setOptions(options.debug)
+    args = parser.parse_args(sys.argv[1:])
+    Options.continueServer |= args.continueServer
+    if args.dbpath:
+        Options.dbPath = os.path.expanduser(args.dbpath)
+    if args.socket:
+        Options.socket = args.socket
+    Debug.setOptions(args.debug)
     Options.fixed = True  # may not be changed anymore
     del parser           # makes Debug.gc quieter
-    return options
+    return args
 
 
-def kajonggServer():
+def kajonggServer() ->None:
     """start the server"""
-    # pylint: disable=too-many-branches
     options = parseArgs()
     if not initDb():
         sys.exit(1)
     realm = MJRealm()
     realm.server = MJServer()
     kajonggPortal = portal.Portal(realm, [DBPasswordChecker()])
-    import predefined  # pylint: disable=unused-variable
+    import predefined
+    predefined.load()
     try:
         if Options.socket:
             # we do not want tracebacks to go from server to client,
             # please check on the server side instead
             factory = pb.PBServerFactory(kajonggPortal, unsafeTracebacks=False)
-            if os.name == 'nt':
+            if sys.platform == 'win32':
                 if Debug.connections:
                     logDebug(
-                        u'local server listening on 127.0.0.1 port %d' %
-                        options.port)
+                        f'local server listening on 127.0.0.1 port {int(options.port)}')
                 reactor.listenTCP(options.port, factory, interface='127.0.0.1')
             else:
                 if Debug.connections:
                     logDebug(
-                        u'local server listening on UNIX socket %s' %
-                        Options.socket)
+                        f'local server listening on UNIX socket {Options.socket}')
                 reactor.listenUNIX(Options.socket, factory)
         else:
             if Debug.connections:
-                logDebug(u'server listening on port %d' % options.port)
+                logDebug(f'server listening on port {int(options.port)}')
             reactor.listenTCP(options.port, pb.PBServerFactory(kajonggPortal))
     except error.CannotListenError as errObj:
         logWarning(errObj)
         sys.exit(1)
     else:
-        reactor.run()
+        reactor.run()  # type:ignore[misc]
 
 
-def profileMe():
+def profileMe() ->None:
     """where do we lose time?"""
     import cProfile
     cProfile.run('kajonggServer()', 'prof')
