@@ -5,98 +5,101 @@ Copyright (C) 2008-2016 Wolfgang Rohdewald <wolfgang@rohdewald.de>
 partially based on C++ code from:
 Copyright (C) 2006 Mauricio Piacentini <mauricio@tabuleiro.com>
 
-Libkmahjongg is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2 of the License, or
-(at your option) any later version.
-
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this program; if not, write to the Free Software
-Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+SPDX-License-Identifier: GPL-2.0
 
 Here we define classes useful for tree views
 """
 
+from typing import Any, List, Sequence, Optional, overload, Union, TYPE_CHECKING
+
 from qt import QAbstractItemModel, QModelIndex
 
+if TYPE_CHECKING:
+    from qt import QObject
+
+
 class TreeItem:
 
     """generic class for items in a tree"""
 
-    def __init__(self, content):
-        self.rawContent = content
-        self.parent = None
-        self.children = []
+    def __init__(self, content: Any) ->None:
+        self.__rawContent = content
+        self.parent:Optional['TreeItem'] = None
+        self.children:List['TreeItem'] = []
 
-    def insert(self, row, child):
+    def insert(self, row:int, child:'TreeItem') ->'TreeItem':
         """add a new child to this tree node"""
         assert isinstance(child, TreeItem)
         child.parent = self
         self.children.insert(row, child)
         return child
 
-    def remove(self):  # pylint: disable=no-self-use
+    def remove(self) ->None:
         """remove this item from the model and the database.
         This is an abstract method."""
-        raise Exception(
+        raise TypeError(
             'cannot remove this TreeItem. We should never get here.')
 
-    def child(self, row):
+    def child(self, row:int) ->'TreeItem':
         """return a specific child item"""
         return self.children[row]
 
-    def childCount(self):
+    def childCount(self) ->int:
         """how many children does this item have?"""
         return len(self.children)
 
-    def content(self, column):
+    def content(self, column:int) ->Any:
         """content held by this item"""
         raise NotImplementedError("Virtual Method")
 
-    def row(self):
+    def row(self) ->int:
         """the row of this item in parent"""
         if self.parent:
             return self.parent.children.index(self)
         return 0
 
+    def columnCount(self) ->int:
+        """Always return 1. Is 1 always correct? No, inherit from RootItem FIXME"""
+        return 1
+
+    @property
+    def raw(self) ->Any:
+        """make it read only"""
+        return self.__rawContent
+
 
 class RootItem(TreeItem):
 
     """an item for header data"""
 
-    def __init__(self, content):
+    def __init__(self, content:Any) ->None:
         TreeItem.__init__(self, content)
 
-    def content(self, column):
+    def content(self, column:int) ->Any:
         """content held by this item"""
-        return self.rawContent[column]
-
-    def columnCount(self):  # pylint: disable=no-self-use
-        """is 1 always correct? No, inherit from RootItem"""
-        return 1
+        return self.raw[column]
 
 
 class TreeModel(QAbstractItemModel):
 
     """a basic class for Kajongg tree views"""
 
-    def columnCount(self, parent):
+    def __init__(self, parent:Optional['QObject']=None) ->None:
+        super().__init__(parent)
+        self.rootItem:Optional[RootItem] = None
+
+    def columnCount(self, parent:QModelIndex=QModelIndex()) ->int:
         """how many columns does this node have?"""
         return self.itemForIndex(parent).columnCount()
 
-    def rowCount(self, parent):
+    def rowCount(self, parent:QModelIndex=QModelIndex()) ->int:
         """how many items?"""
         if parent.isValid() and parent.column():
             # all children have col=0 for parent
             return 0
         return self.itemForIndex(parent).childCount()
 
-    def index(self, row, column, parent):
+    def index(self, row:int, column:int, parent:QModelIndex=QModelIndex()) ->QModelIndex:
         """generate an index for this item"""
         if self.rootItem is None:
             return QModelIndex()
@@ -112,8 +115,14 @@ class TreeModel(QAbstractItemModel):
             return self.createIndex(row, column, item)
         return QModelIndex()
 
-    def parent(self, index):
-        """find the parent index"""
+    @overload
+    def parent(self, index:QModelIndex) ->QModelIndex: ...
+    @overload
+    def parent(self) ->'QObject': ...
+
+    def parent(self, index:Optional[QModelIndex]=None) ->Union[QModelIndex, 'QObject']:
+        """find the parent for index"""
+        assert isinstance(index, QModelIndex)  # we never use parent() for getting QObject
         if not index.isValid():
             return QModelIndex()
         childItem = self.itemForIndex(index)
@@ -127,15 +136,17 @@ class TreeModel(QAbstractItemModel):
                         return self.createIndex(row, 0, parentItem)
         return QModelIndex()
 
-    def itemForIndex(self, index):
-        """returns the item at index"""
+    def itemForIndex(self, index:QModelIndex) ->TreeItem:
+        """return the item at index"""
         if index.isValid():
             item = index.internalPointer()
             if item:
                 return item
+        assert self.rootItem
         return self.rootItem
 
-    def insertRows(self, position, items, parent=QModelIndex()):
+    def insertRows(self, position:int, items:Sequence[TreeItem],  # type:ignore[override]
+        parent:QModelIndex=QModelIndex()) ->bool:
         """inserts items into the model"""
         parentItem = self.itemForIndex(parent)
         self.beginInsertRows(parent, position, position + len(items) - 1)
@@ -144,7 +155,7 @@ class TreeModel(QAbstractItemModel):
         self.endInsertRows()
         return True
 
-    def removeRows(self, position, rows=1, parent=QModelIndex()):
+    def removeRows(self, position:int, rows:int=1, parent:QModelIndex=QModelIndex()) ->bool:
         """reimplement QAbstractItemModel.removeRows"""
         self.beginRemoveRows(parent, position, position + rows - 1)
         parentItem = self.itemForIndex(parent)
